
#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


from naruno.wallet.wallet_import import wallet_import, Address
from naruno.lib.log import get_logger

from naruno.transactions.transaction import Transaction

import messaging_app
import time

def messaging_app_main_tx(tx):

    logger = get_logger("Messaging_App")
    logger.info(
                f"A transaction sended to messaging app: {tx.__dict__}"
            )
    my_pubkey = wallet_import(-1, 3)
    

    control = False
    to_User = False

    
    if tx.toUser == my_pubkey:
        control = True
        to_User = True
    logger.debug(
                f"control: {control} | to_User: {to_User}"
            )
    if control:
     if tx.data != None:
      logger.debug(f"tx.data: {tx.data}")
      if "app" in tx.data:
       
            if "addnewusern" in tx.data["action"] and to_User:
                from messaging_app.func.create_new_user import create_new_user
                create_new_user(tx.fromUser[:15], tx.fromUser, tx.data["app_data"], 0)
            elif "addnewusere" in tx.data["action"] and to_User:
                from messaging_app.func.create_new_user import create_new_user
                create_new_user(tx.fromUser[:15], tx.fromUser, 0,tx.data["app_data"])                
            elif "newmessage" in tx.data["action"] and to_User:
                from messaging_app.func.decrypt import decrypt_text
                decrypt_text(tx.data["app_data"],tx.fromUser)
                

def get_thread():
    while True:
        new_datas = messaging_app.func.integration.the_integration.get()
        if new_datas != []:
            for data in new_datas:
                messaging_app_main_tx(Transaction(data["sequence_number"], data["signature"], data["fromUser"], data["toUser"], data["data"], data["amount"], data["transaction_fee"], data["transaction_time"]))
        time.sleep(5)






from messaging_app.lib.keys_system import the_keys, save_keys

import os

def encrypt_text(text_data,key):
    temp_keys = the_keys()[key]
    n = temp_keys["n"]                               #declare n
    e = temp_keys["e"]                               #declare e

    encrypted_mess = ""     #create 
    for word in text_data:
        for char in word:
            encrypted_mess += str(encryption(ord(char),e,n))+"\n"   #encrypt by char and write to file

    encrypted_mess += str(n)+"\n"                             #print n and e
    encrypted_mess += str(e)

    return(encrypted_mess)

#encyrption funciton. m^e % n
def encryption(m,e,n):
    x = pow(m,e,n)
    return x



try:
    from flask import Flask, render_template, request, redirect
except:
    import pip
    package = 'flask'
    if hasattr(pip, 'main'):
        pip.main(['install', package])
    else:
        pip._internal.main(['install', package])

import os
import sys

from waitress import serve

from messaging_app.lib.keys_system import the_keys
from messaging_app.lib.messages_system import  the_message
from messaging_app.func.send import add_new_user_request, send_new_message

from messaging_app.func.encrypt import encrypt_text


from messaging_app.func.save_new_message import save_new_message


from naruno.apps.remote_app import Integration

from messaging_app.func.get import get_thread

import messaging_app

app = Flask(__name__)
from messaging_app.func.integration import the_integration

@app.route('/')
def main_page():
    return render_template('home.html')
@app.route('/user-list')
def user_list():
    users = []

    temp_keys = the_keys()

    for element in temp_keys:
        if "name" in temp_keys[element]:
            users.append(temp_keys[element])
    

    return render_template('user-list.html', users=users)

@app.route('/messages/<name>')
def messages(name):

    temp_keys = the_keys()

    for element in temp_keys:
        if "name" in temp_keys[element]:
          if temp_keys[element]["name"] == name:
            return render_template('messages.html', user_id=element)

@app.route('/get-messages/<id>')
def get_messages(id):

    temp_keys = the_keys()

    if str(id) in temp_keys:
        messages = the_message(str(id))
        return render_template('messages-col.html', user=temp_keys[str(id)], messages=messages)

@app.route("/add-new-user", methods =['POST','GET'])
def add_new_users():
    if request.method == 'POST':
 
        publickey = request.form.get('publickey').replace("\n","")
        add_new_user_request(publickey)
 
        return redirect(request.referrer)

@app.route("/new-message", methods =['POST','GET'])
def send_message():
    if request.method == 'POST':
 
        user = request.form.get('user')

        text = request.form.get('text')


        temp_keys = the_keys()[user]

        encrypted = encrypt_text(text, user)

        print("\n**** encrypted: "+ encrypted)

        pubkey = temp_keys["fromUser"]

        print("\n**** pubkey: "+ pubkey)


        send_new_message(encrypted, pubkey)

        message = {"from":"me","message":text}

        
        

        save_new_message(message,temp_keys["n"],temp_keys["e"])
 
        return redirect(request.referrer)

def start_messaging_app(port=81, integration_port=8000, password="123"):
    messaging_app.func.integration.the_integration = Integration("messaging_app", port=integration_port, password=password, sended=True)
    from threading import Thread
    t = Thread(target=get_thread)
    t.start()
    serve(app, host="0.0.0.0", port=port)





#!/usr/bin/python3
# -*- coding: utf-8 -*-
import unittest


class Test_rsa(unittest.TestCase):

    def test_creating_and_delete_rsa(self):
        temp_rsa = create_new_rsa()

        self.assertEqual(temp_rsa in the_keys(),True,"A problem on the test_creating_rsa")

        key_delete(temp_rsa)

        self.assertEqual(temp_rsa not in the_keys(),True,"A problem on the test_creating_rsa")







import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../../.."))
from messaging_app.func.create_new_rsa import create_new_rsa
from messaging_app.func.encrypt import encrypt_text
from messaging_app.func.decrypt import decrypt_text
from messaging_app.lib.keys_system import the_keys, key_delete
unittest.main(exit=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import unittest


class Test_Message(unittest.TestCase):


    def test_create_and_saving_and_deleting(self):
   
        

        name = "onur"
        fromUser = "fdghfhgf"

        n = 111
        e = 111


        temp_user = create_new_user(name,fromUser,n,e)

        message = {"from":"me","message":"hello"}

        number = save_new_message(message,n,e)


        #save_new_message({"from":"sender","message":"hello how are you"},n,e)

        ok = False

        temp_message_object = the_message(temp_user) 

        if number in temp_message_object:
          if temp_message_object[number] == message:
            ok = True



        key_delete(temp_user)

        message_delete(temp_user)



        self.assertEqual(ok,True,"A problem on the test_encrypt_decrypt")






import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../../.."))

from messaging_app.func.create_new_rsa import create_new_rsa
from messaging_app.func.create_new_user import create_new_user
from messaging_app.lib.keys_system import the_keys, key_delete
from messaging_app.lib.messages_system import the_message, message_delete
from messaging_app.func.save_new_message import save_new_message
unittest.main(exit=False)



from messaging_app.func.gen_keys import gen_keys
from messaging_app.lib.keys_system import the_keys, save_keys

from naruno.wallet.wallet_import import Address, wallet_import

def create_new_user(name,fromUser,n,e):

    
    temp_keys = the_keys()

    in_list = False
    if not fromUser == wallet_import(-1, 3):
        for key in temp_keys:
          if "fromUser" in temp_keys[key]:
            if temp_keys[key]["fromUser"] == fromUser:
                in_list = True
                if temp_keys[key]["n"] == 0 and temp_keys[key]["e"] == 0:
                    temp_keys[key]["n"] = n
                    temp_keys[key]["e"] = e
                    save_keys(temp_keys)
                    return(key)

    if not in_list:
        number = str(len(temp_keys) + 1)
        
        temp_keys[number] = {}
        
        temp_keys[number]["name"] = name

        temp_keys[number]["fromUser"] = fromUser
        temp_keys[number]["n"] = n
        temp_keys[number]["e"] = e

        save_keys(temp_keys)
        
        return(number)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import unittest


class Test_User(unittest.TestCase):


    def test_create_and_saving_and_deleting(self):
   
        

        name = "onur"

        pubkey = "onurspubkey"

        n = 111
        e = 111


        temp_user = create_new_user(name,pubkey,n,e)

        temp_user_object = the_keys()[temp_user]

        
        ok = False
        if temp_user_object["name"] == name and temp_user_object["n"] == n and temp_user_object["e"] == e:
            ok = True


        key_delete(temp_user)



        self.assertEqual(ok,True,"A problem on the test_encrypt_decrypt")





import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../../.."))
from messaging_app.func.create_new_rsa import create_new_rsa
from messaging_app.func.create_new_user import create_new_user
from messaging_app.lib.keys_system import the_keys, key_delete
unittest.main(exit=False)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import json

import os
from messaging_app.func.create_new_rsa import create_new_rsa

from messaging_app.messaging_app_config import KEY_PATH, KEY_BIT

def save_keys(new_keys):
    print("saving: "+str(new_keys))
    with open(KEY_PATH, 'w') as keys_file:
        json.dump(new_keys, keys_file)

def keys_class():
    temp_json = {}
    save_keys(temp_json)
    return(temp_json)




def the_keys():

    if not os.path.exists(KEY_PATH):
        keys_class()
        create_new_rsa()

    with open(KEY_PATH, 'rb') as keys_file:
        return json.load(keys_file)

def key_delete(account):
    saved_keys = the_keys()
    if account in saved_keys:
        del saved_keys[account]
        save_keys(saved_keys)



import math
import random

from messaging_app.func.miller_rabin import is_prime



def get_prime(bit):
    while True:
        num = random.randrange(2**(bit-1), 2**(bit))#random.getrandbits(bit)
        if is_prime(num):
           return num       



def gen_keys(bit):
    p = get_prime(bit)
    #print("p = ",p)
    q = get_prime(bit)
    #print("q = ",q)
    n = p * q
    m = (p - 1) * (q - 1)
    e = get_e(m,bit)
    d = get_d(e, m)
    while d < 0:
        d += m
    return [p, q, n, e, d]


def get_e(m,bit):
    """Finds an e coprime with m."""
    #e = 2
    e = get_prime(bit)
    while gcd(e, m) != 1:
        e += 1
    return e


def gcd(a, b):
    """Euclid's Algorithm: Takes two integers and returns gcd."""
    while b > 0:
        a, b = b, a % b
    return a

def get_d(e, m):
    x = lasty = 0
    lastx = y = 1
    while m != 0:
        q = e // m
        e, m = m, e % m
        x, lastx = lastx - q*x, x
        y, lasty = lasty - q*y, y
    return lastx




from messaging_app.lib.keys_system import the_keys

from messaging_app.lib.messages_system import the_message, save_message

def save_new_message(message,n,e):

    temp_keys = the_keys()




    for temp_element in temp_keys:
        print("\n\n\n")
        print(temp_keys[temp_element]["n"] == n and temp_keys[temp_element]["e"] == e)
        if temp_keys[temp_element]["n"] == n and temp_keys[temp_element]["e"] == e and "d" not in temp_keys[temp_element]:
            
            temp_message = the_message(temp_element)
            
            number = str(len(temp_message) + 1)

            temp_message[number] = message
            
            save_message(temp_message,temp_element)
            
            return(number)





from messaging_app.lib.keys_system import the_keys



from messaging_app.func.create_new_user import create_new_user

import messaging_app
from messaging_app.func.integration import the_integration

def add_new_user_request(publickey):
    key = the_keys()["1"]


    messaging_app.func.integration.the_integration.send("addnewusern", key["n"], publickey)
    messaging_app.func.integration.the_integration.send("addnewusere", key["3"], publickey)


    create_new_user(publickey[:10] ,publickey,0,0)

def send_new_message(message,publickey):
    messaging_app.func.integration.the_integration.send("newmessage", message, publickey)



from .web.chat import start_messaging_app

from .func.create_new_rsa import create_new_rsa
from .func.create_new_user import create_new_user
from .lib.keys_system import the_keys, key_delete
from .lib.messages_system import the_message, message_delete
from .func.save_new_message import save_new_message


from messaging_app.func.gen_keys import gen_keys
from messaging_app.messaging_app_config import KEY_BIT


def create_new_rsa():

    p,q,n,e,d = gen_keys(KEY_BIT)
    
    from messaging_app.lib.keys_system import the_keys, save_keys
    temp_keys = the_keys()

    number = str(len(temp_keys) + 1)

    temp_keys[number] = {}

    temp_keys[number]["p"] = p
    temp_keys[number]["q"] = q
    temp_keys[number]["n"] = n
    temp_keys[number]["e"] = e
    temp_keys[number]["d"] = d




    # Public key(n + e)
    # Private key(n + d)



    save_keys(temp_keys)

    return(number)



from messaging_app.lib.keys_system import the_keys, save_keys

from messaging_app.func.save_new_message import save_new_message

from wallet.wallet import Address, wallet_import

def decrypt_text(text_data,pubkey):

    nums = []

    for line in text_data.splitlines():
        nums.append(int(line))
    l = len(nums)
    n = nums[l-2]
    e = nums[l-1]

    print("\n\n\n e: "+str(e))
    

    temp_keys = the_keys()

    print(temp_keys)
    

    d = temp_keys["1"]["d"]
    
    if not pubkey == wallet_import(-1, 3):
        for keys in temp_keys:
          if "fromUser" in temp_keys[keys]:
            key_publickey = temp_keys[keys]["fromUser"].replace("\n","").replace(" ","")
            if pubkey == key_publickey:
                i = 0
                decrypted_Mess = ""
                while i < l -2:
                    x = decrypt(nums[i], int(d), int(n))
                    y = chr(x)
                    decrypted_Mess += y
                    i+=1
                message = {"from":"user","message":decrypted_Mess}
                save_new_message(message,temp_keys[keys]["n"],temp_keys[keys]["e"])
                return decrypted_Mess


def decrypt(c,d,n):
    x = pow(c,d,n)
    return x



from setuptools import setup


setup(name='messaging_app',
version='0.1.2',
description="""Messaging App""",
long_description_content_type="text/markdown",
long_description="".join(open("README.md", encoding="utf-8").readlines()),
url='https://github.com/Naruno/Messaging-App',
author="Naruno Developers",
author_email='onur.atakan.ulusoy@naruno.org',
license='MIT',
packages=["messaging_app"],
include_package_data=True,
entry_points = {
    'console_scripts': ['messagingapp=messaging_app.messaging_app:main'],
},
python_requires=">= 3",
zip_safe=False)



from messaging_app.func.gen_keys import gen_keys
from lib.keys_system import the_keys, save_keys
from lib.users_system import save_user

def create_new_user(n,e):

    temp_keys = the_keys()

    number = str(len(temp_keys) + 1)
    
    temp_message = {}

    temp_message["n"] = n
    temp_message["e"] = e


    save_user(temp_message,number)

    return(number)


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import json

import os


from messaging_app.messaging_app_config import MESSAGES_PATH

def save_message(new_messages,account):
    

    temp_path = (MESSAGES_PATH+str(account)+".json")

    with open(temp_path, 'w') as messages_file:
        json.dump(new_messages, messages_file)

def message_class(account):
    temp_json = {}
    save_message(temp_json,account)
    return(temp_json)




def the_message(account):
    

    temp_path = (MESSAGES_PATH+str(account)+".json")

    if not os.path.exists(temp_path):
        return message_class(account)

    with open(temp_path, 'rb') as messages_file:
        return json.load(messages_file)

def message_delete(account):
    temp_path = MESSAGES_PATH+str(account)+".json"
    if os.path.exists(temp_path):
        os.remove(temp_path)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import argparse

from messaging_app.messaging_app_config import password

import messaging_app

def main():
    parser = argparse.ArgumentParser(
        description="RSA based blockchain messaging app.")

    parser.add_argument("-pass",
                        "--password",
                        default="123",
                        type=str,
                        help="Give the naruno pass")


    parser.add_argument("-p",
                        "--port",
                        default=81,
                        type=int,
                        help="Give a port for api")

    parser.add_argument("-ip",
                        "--integrationport",
                        default=8000,
                        type=int,
                        help="Give a port for integratin")

    

    args = parser.parse_args()
    messaging_app.messaging_app_config.password = args.password

    from messaging_app.web.chat import start_messaging_app
    start_messaging_app(port=args.port, integration_port=args.integrationport, password=args.password)

